/*********************************************
 * OPL 12.3 Model
 * Author: mireia.roca
 * Creation Date: 22/02/2012
 *********************************************/
/* Problem Description:
 * Scheduling Tasks with Time Windows to Solve the Scheduling of Booked Parking
 * Objective Function: (3) Minimize the number of rescheduled tasks 
   * T limits in the day [0,1440] */
 
 // Parameters & Sets (DATA)
 
 int Id = ...; // Identifier of the problem set
 int c = ...; // number of loading areas
 int n = ...; // number of total requests
 range RQ=1..n; // set of requests
 float td[RQ]=...; //Aproximated time of Service for each RQ
 int a[RQ]=...; // Begining of TW
 int b[RQ]=...; // Ending of TW
 
 int M=1500; // To avoid nonlinearity. It should be adjusted to the value 10e30 gives errors
 int K=1500; //  To make that variable bet be integer defined by Tardy and Early constrictions
 
 range N= 0..n+1; // Nodes with origin/destionation depot
 
 tuple arcs{
   int orig;
   int dest;
 }   
 
 {arcs} DI={<i,j>| i in N, j in RQ: i==0};
 {arcs} DF={<i,j>| i in RQ, j in N: j==n+1};
 {arcs} DD={<i,j>| i in RQ, j in RQ: i==j};
 {arcs} V={<i,j>| i in RQ, j in RQ};
 {arcs} VU=V union DI union DF diff DD;
 
  //  Variables 
 
 dvar boolean x[VU]; // Arcs of consecutive requests equal 1, 0 otherwise
 dvar float T[RQ];   // Time of begining serving the request
 dvar float et[RQ];		 // Earliness Tardiness Penalty
 dvar boolean bet[RQ];		 // Is the task performed on time? 0 yes 1 no. Earliness Tardiness Penalty

 
 // Objective
 minimize sum(j in RQ) bet[j];
 
 // Constraints
 subject to{ 
 
	Places: 
	sum(j in RQ) x[<j,n+1>]<=c;
	
	Places2:
	sum(j in RQ) x[<0,j>]<=c;
	
	forall(j in RQ)
	Connectivity:
	sum(<j,i> in VU) x[<j,i>]-sum(<i,j> in VU) x[<i,j>] == 0;
	
	forall(<i,j> in V: i!=j)
	Time:
	(T[i]+td[i]-T[j]) <= (1-x[<i,j>])*M;
	
	//forall(i in RQ)
	//TimeWindowa:
	//a[i]<=T[i];
	
	//forall(i in RQ)
	//TimeWindowb:
	//T[i]<=b[i];
	
	forall(i in RQ)
	JobCompletion:
	sum(<i,j> in VU) x[<i,j>]==1;

	forall(i in RQ)
	Positive:
	et[i]>=0;
	
	forall(i in RQ)
	Earliness:
	et[i]>=a[i]-T[i];

	forall(i in RQ)
	Tardiness:
	et[i]>=T[i]-b[i];	
	
	forall(i in RQ)
	ReschedTask:
	bet[i]*K>=et[i];

	forall(i in RQ)
	PositiveT:
	T[i]>=0;
	
	forall(i in RQ)
	MaxT:
	T[i]<=1440;	
} 