/*********************************************
 * OPL 12.3 Model
 * Author: mireia.roca
 * Creation Date: 10/02/2012
 *********************************************/
/* Problem Description:
 * Scheduling Tasks with Time Windows to Solve the Scheduling of Booked Parking
 * Objective Function: (4) Minimize earliness-tardiness subject to Maximum displacement (Model 1 + new restrictions)  
 * T limits in the day [0,1440] 
 * M is 600 due that maxdis restriction*/
 
 // Parameters & Sets (DATA)
 
 int Id = ...; // Identifier of the problem set
 int c = ...; // number of loading areas
 int n = ...; // number of total requests
 range RQ=1..n; // set of requests
 float td[RQ]=...; //Aproximated time of Service for each RQ
 int a[RQ]=...; // Begining of TW
 int b[RQ]=...; // Ending of TW
 int maxdis = 60;// Maximum lateness or earliness
 
 int M=600; // To avoid nonlinearity. It should be adjusted to the value 10e30 gives errors
 
 range N= 0..n+1; // Nodes with origin/destionation depot
 
 tuple arcs{
   int orig;
   int dest;
 }   
 
 {arcs} DI={<i,j>| i in N, j in RQ: i==0};
 {arcs} DF={<i,j>| i in RQ, j in N: j==n+1};
 {arcs} DD={<i,j>| i in RQ, j in RQ: i==j};
 {arcs} V={<i,j>| i in RQ, j in RQ};
 {arcs} VU=V union DI union DF diff DD;
 
  //  Variables 
 
 dvar boolean x[VU]; // Arcs of consecutive requests equal 1, 0 otherwise
 dvar float T[RQ];   // Time of begining serving the request
 dvar float et[RQ];		 // Earliness Tardiness Penalty
 
 // Objective
 minimize sum(j in RQ) et[j];
 
 // Constraints
 subject to{ 
 
	Places: 
	sum(j in RQ) x[<j,n+1>]<=c;
	
	Places2:
	sum(j in RQ) x[<0,j>]<=c;
	
	forall(j in RQ)
	Connectivity:
	sum(<j,i> in VU) x[<j,i>]-sum(<i,j> in VU) x[<i,j>] == 0;
	
	forall(<i,j> in V: i!=j)
	Time:
	(T[i]+td[i]-T[j]) <= (1-x[<i,j>])*M;
	
	//forall(i in RQ)
	//TimeWindowa:
	//a[i]<=T[i];
	
	//forall(i in RQ)
	//TimeWindowb:
	//T[i]<=b[i];
	
	forall(i in RQ)
	JobCompletion:
	sum(<i,j> in VU) x[<i,j>]==1;
	
	forall(i in RQ)
	Positive:
	et[i]>=0;
	
	forall(i in RQ)
	Earliness:
	et[i]>=a[i]-T[i];

	forall(i in RQ)
	Tardiness:
	et[i]>=T[i]-b[i];
	
	forall(i in RQ)
	MaxDispl:
	et[i]<=maxdis;

	forall(i in RQ)
	PositiveT:
	T[i]>=0;
	
	forall(i in RQ)
	MaxT:
	T[i]<=1440;	
} 